# Generated by roxygen2: do not edit by hand

S3method("%*%",estimate)
S3method("%++%","function")
S3method("%++%",character)
S3method("%++%",default)
S3method("%++%",estimate)
S3method("%++%",lvm)
S3method("%++%",matrix)
S3method("*",estimate)
S3method("+",estimate)
S3method("+",lvm)
S3method("-",estimate)
S3method("/",estimate)
S3method("==",estimate)
S3method("Graph<-",lvm)
S3method("Graph<-",lvmfit)
S3method("Model<-",lvm)
S3method("Model<-",lvmfit)
S3method("Model<-",multigroup)
S3method("Model<-",multigroupfit)
S3method("[",estimate)
S3method("[",sim)
S3method("^",estimate)
S3method("addvar<-",lvm)
S3method("cancel<-",lvm)
S3method("constrain<-",default)
S3method("constrain<-",multigroup)
S3method("constrain<-",multigroupfit)
S3method("covariance<-",lvm)
S3method("covfix<-",lvm)
S3method("distribution<-",lvm)
S3method("edgelabels<-",graphNEL)
S3method("edgelabels<-",lvm)
S3method("edgelabels<-",lvmfit)
S3method("exogenous<-",lvm)
S3method("functional<-",lvm)
S3method("heavytail<-",lvm)
S3method("index<-",lvm)
S3method("index<-",lvmfit)
S3method("intercept<-",lvm)
S3method("intervention<-",lvm)
S3method("intfix<-",lvm)
S3method("kill<-",lvm)
S3method("labels<-",default)
S3method("latent<-",lvm)
S3method("nodecolor<-",default)
S3method("nodecolor<-",lvm)
S3method("nonlinear<-",lvm)
S3method("ordinal<-",lvm)
S3method("parameter<-",lvm)
S3method("parameter<-",lvmfit)
S3method("parfix<-",lvm)
S3method("randomslope<-",lvm)
S3method("regfix<-",lvm)
S3method("regression<-",lvm)
S3method("rmvar<-",lvm)
S3method("transform<-",lvm)
S3method("variance<-",lvm)
S3method(Graph,lvm)
S3method(Graph,lvmfit)
S3method(IC,data.frame)
S3method(IC,default)
S3method(IC,estimate)
S3method(IC,estimate.complik)
S3method(IC,estimate.list)
S3method(IC,glm)
S3method(IC,lvm.mixture)
S3method(IC,matrix)
S3method(IC,mlm)
S3method(IC,multigroupfit)
S3method(IC,multinomial)
S3method(IC,numeric)
S3method(IC,twostageCV)
S3method(Model,default)
S3method(Model,lvm)
S3method(Model,lvmfit)
S3method(Model,multigroup)
S3method(Model,multigroupfit)
S3method(Model,twostageCV)
S3method(Weights,default)
S3method(acos,estimate)
S3method(acosh,estimate)
S3method(addattr,graphNEL)
S3method(addattr,lvm)
S3method(addattr,lvmfit)
S3method(addvar,lvm)
S3method(adjMat,lvm)
S3method(adjMat,lvmfit)
S3method(ancestors,lvm)
S3method(ancestors,lvmfit)
S3method(as.matrix,sim)
S3method(as.numeric,list)
S3method(as.vector,sim)
S3method(asin,estimate)
S3method(asinh,estimate)
S3method(atan,estimate)
S3method(atanh,estimate)
S3method(baptize,lvm)
S3method(bootstrap,lvm)
S3method(bootstrap,lvmfit)
S3method(c,estimate)
S3method(cancel,lvm)
S3method(cbind,sim)
S3method(children,lvm)
S3method(children,lvmfit)
S3method(click,default)
S3method(coef,effects)
S3method(coef,estimate)
S3method(coef,estimate.list)
S3method(coef,lvm)
S3method(coef,lvm.mixture)
S3method(coef,lvmfit)
S3method(coef,multigroup)
S3method(coef,multigroupfit)
S3method(coef,multinomial)
S3method(coef,ordreg)
S3method(coef,pcor)
S3method(coef,summary.estimate)
S3method(coef,summary.lvmfit)
S3method(coef,twostageCV)
S3method(coef,zibreg)
S3method(compare,default)
S3method(confint,effects)
S3method(confint,lvmfit)
S3method(confint,multigroupfit)
S3method(constrain,default)
S3method(correlation,data.frame)
S3method(correlation,lvmfit)
S3method(correlation,matrix)
S3method(cos,estimate)
S3method(cosh,estimate)
S3method(covariance,formula)
S3method(covariance,lvm)
S3method(covfix,lvm)
S3method(density,sim)
S3method(deriv,"function")
S3method(deriv,lvm)
S3method(descendants,lvm)
S3method(descendants,lvmfit)
S3method(distribution,lvm)
S3method(dsep,lvm)
S3method(edgeList,lvm)
S3method(edgeList,lvmfit)
S3method(edgelabels,graphNEL)
S3method(edgelabels,lvm)
S3method(edgelabels,lvmfit)
S3method(effects,lvmfit)
S3method(endogenous,list)
S3method(endogenous,lm)
S3method(endogenous,lvm)
S3method(endogenous,lvmfit)
S3method(endogenous,multigroup)
S3method(estimate,MAR)
S3method(estimate,array)
S3method(estimate,data.frame)
S3method(estimate,default)
S3method(estimate,formula)
S3method(estimate,glm)
S3method(estimate,list)
S3method(estimate,lvm)
S3method(estimate,mlm)
S3method(estimate,multigroup)
S3method(estimate,twostage.lvm)
S3method(exogenous,list)
S3method(exogenous,lm)
S3method(exogenous,lvm)
S3method(exogenous,lvmfit)
S3method(exogenous,multigroup)
S3method(exp,estimate)
S3method(expm1,estimate)
S3method(family,zibreg)
S3method(finalize,lvm)
S3method(formula,lvm)
S3method(formula,lvmfit)
S3method(functional,lvm)
S3method(gof,lvmfit)
S3method(heavytail,lvm)
S3method(iid,default)
S3method(index,lvm)
S3method(index,lvmfit)
S3method(influence,estimate)
S3method(information,data.frame)
S3method(information,glm)
S3method(information,lvm)
S3method(information,lvm.missing)
S3method(information,lvm.mixture)
S3method(information,lvmfit)
S3method(information,multigroup)
S3method(information,multigroupfit)
S3method(information,multinomial)
S3method(information,table)
S3method(information,zibreg)
S3method(intercept,lvm)
S3method(intervention,lvm)
S3method(kappa,data.frame)
S3method(kappa,multinomial)
S3method(kappa,table)
S3method(kill,lvm)
S3method(labels,estimate)
S3method(labels,graphNEL)
S3method(labels,lvm)
S3method(labels,lvmfit)
S3method(latent,list)
S3method(latent,lvm)
S3method(latent,lvmfit)
S3method(latent,multigroup)
S3method(log,estimate)
S3method(log1p,estimate)
S3method(logLik,estimate.complik)
S3method(logLik,lvm)
S3method(logLik,lvm.missing)
S3method(logLik,lvm.mixture)
S3method(logLik,lvmfit)
S3method(logLik,multigroup)
S3method(logLik,multigroupfit)
S3method(logLik,ordreg)
S3method(logLik,pcor)
S3method(logLik,zibreg)
S3method(manifest,list)
S3method(manifest,lvm)
S3method(manifest,lvm.mixture)
S3method(manifest,lvmfit)
S3method(manifest,multigroup)
S3method(matrices,lvm)
S3method(matrices,multigroup)
S3method(merge,estimate)
S3method(merge,estimate.list)
S3method(merge,glm)
S3method(merge,list)
S3method(merge,lm)
S3method(merge,lvm)
S3method(merge,lvmfit)
S3method(merge,multinomial)
S3method(model.frame,estimate)
S3method(model.frame,lvm.mixture)
S3method(model.frame,lvmfit)
S3method(model.frame,multigroupfit)
S3method(model.frame,multinomial)
S3method(modelPar,lvm)
S3method(modelPar,lvmfit)
S3method(modelPar,multigroup)
S3method(modelPar,multigroupfit)
S3method(modelVar,lvm)
S3method(modelVar,lvmfit)
S3method(moments,lvm)
S3method(moments,lvm.missing)
S3method(moments,lvmfit)
S3method(nonlinear,lvm)
S3method(nonlinear,lvmfit)
S3method(nonlinear,twostage.lvm)
S3method(ordinal,lvm)
S3method(parameter,default)
S3method(parameter,estimate)
S3method(parents,lvm)
S3method(parents,lvmfit)
S3method(parfix,lvm)
S3method(parpos,default)
S3method(parpos,lvm)
S3method(parpos,lvmfit)
S3method(parpos,multigroup)
S3method(parpos,multigroupfit)
S3method(pars,default)
S3method(pars,glm)
S3method(pars,lvm)
S3method(pars,lvm.missing)
S3method(pars,mlm)
S3method(pars,survreg)
S3method(path,graphNEL)
S3method(path,lvm)
S3method(path,lvmfit)
S3method(plot,estimate)
S3method(plot,lvm)
S3method(plot,lvm.mixture)
S3method(plot,lvmfit)
S3method(plot,multigroup)
S3method(plot,multigroupfit)
S3method(plot,mvn.mixture)
S3method(plot,sim)
S3method(plot,twostage.lvm)
S3method(predict,lvm)
S3method(predict,lvm.missing)
S3method(predict,lvm.mixture)
S3method(predict,lvmfit)
S3method(predict,multinomial)
S3method(predict,ordreg)
S3method(predict,twostage.lvmfit)
S3method(predict,twostageCV)
S3method(predict,zibreg)
S3method(print,Combine)
S3method(print,bootstrap.lvm)
S3method(print,diagtest)
S3method(print,effects)
S3method(print,equivalence)
S3method(print,estimate)
S3method(print,fix)
S3method(print,gkgamma)
S3method(print,gof.lvmfit)
S3method(print,lvm)
S3method(print,lvm.mixture)
S3method(print,lvm.predict)
S3method(print,lvmfit)
S3method(print,lvmfit.randomslope)
S3method(print,modelsearch)
S3method(print,multigroup)
S3method(print,multigroupfit)
S3method(print,multinomial)
S3method(print,mvn.mixture)
S3method(print,offdiag)
S3method(print,ordinal.lvm)
S3method(print,ordreg)
S3method(print,pcor)
S3method(print,sim)
S3method(print,summary.estimate)
S3method(print,summary.lvm.mixture)
S3method(print,summary.lvmfit)
S3method(print,summary.multigroupfit)
S3method(print,summary.ordreg)
S3method(print,summary.sim)
S3method(print,summary.twostageCV)
S3method(print,summary.zibreg)
S3method(print,test_adj)
S3method(print,transform.lvm)
S3method(print,twostage.lvm)
S3method(print,twostageCV)
S3method(print,zibreg)
S3method(prod,estimate)
S3method(profile,lvmfit)
S3method(randomslope,lvm)
S3method(randomslope,lvmfit)
S3method(rbind,Surv)
S3method(rbind,sim)
S3method(regfix,lvm)
S3method(regression,formula)
S3method(regression,lvm)
S3method(residuals,lvm)
S3method(residuals,lvmfit)
S3method(residuals,multigroupfit)
S3method(residuals,zibreg)
S3method(rmvar,lvm)
S3method(roots,lvm)
S3method(roots,lvmfit)
S3method(score,estimate.complik)
S3method(score,glm)
S3method(score,lm)
S3method(score,lvm)
S3method(score,lvm.missing)
S3method(score,lvm.mixture)
S3method(score,lvmfit)
S3method(score,multigroup)
S3method(score,multigroupfit)
S3method(score,ordreg)
S3method(score,pcor)
S3method(score,survreg)
S3method(score,zibreg)
S3method(sim,default)
S3method(sim,lvm)
S3method(sim,lvmfit)
S3method(sim,mvn.mixture)
S3method(simulate,lvm)
S3method(simulate,lvmfit)
S3method(sin,estimate)
S3method(sinh,estimate)
S3method(sinks,lvm)
S3method(sinks,lvmfit)
S3method(sqrt,estimate)
S3method(stack,estimate)
S3method(stack,glm)
S3method(stack,lvmfit)
S3method(subset,estimate)
S3method(subset,lvm)
S3method(sum,estimate)
S3method(summary,diagtest)
S3method(summary,effects)
S3method(summary,estimate)
S3method(summary,lvm)
S3method(summary,lvm.mixture)
S3method(summary,lvmfit)
S3method(summary,multigroup)
S3method(summary,multigroupfit)
S3method(summary,ordreg)
S3method(summary,sim)
S3method(summary,test_adj)
S3method(summary,twostageCV)
S3method(summary,zibreg)
S3method(tan,estimate)
S3method(tanh,estimate)
S3method(totaleffects,lvmfit)
S3method(tr,matrix)
S3method(transform,estimate)
S3method(transform,lvm)
S3method(twostage,estimate)
S3method(twostage,lvm)
S3method(twostage,lvm.mixture)
S3method(twostage,lvmfit)
S3method(twostage,twostage.lvm)
S3method(variance,formula)
S3method(variance,lvm)
S3method(vars,graph)
S3method(vars,list)
S3method(vars,lm)
S3method(vars,lvm)
S3method(vars,lvmfit)
S3method(vcov,effects)
S3method(vcov,estimate)
S3method(vcov,estimate.list)
S3method(vcov,lvm.mixture)
S3method(vcov,lvmfit)
S3method(vcov,multigroupfit)
S3method(vcov,multinomial)
S3method(vcov,ordreg)
S3method(vcov,pcor)
S3method(vcov,twostageCV)
S3method(vcov,zibreg)
S3method(with,estimate)
export("%++%")
export("%in.closed%")
export("%in.open%")
export("%ni%")
export("Graph<-")
export("Missing<-")
export("Model<-")
export("addvar<-")
export("cancel<-")
export("categorical<-")
export("constrain<-")
export("covariance<-")
export("distribution<-")
export("edgelabels<-")
export("eventTime<-")
export("exogenous<-")
export("functional<-")
export("heavytail<-")
export("index<-")
export("intercept<-")
export("intervention<-")
export("kill<-")
export("labels<-")
export("latent<-")
export("nodecolor<-")
export("nonlinear<-")
export("offdiag<-")
export("ordinal<-")
export("parameter<-")
export("randomslope<-")
export("regression<-")
export("revdiag<-")
export("rmvar<-")
export("timedep<-")
export("transform<-")
export("variance<-")
export(Binary.lvm)
export(By)
export(CoefMat)
export(CoefMat.multigroupfit)
export(Col)
export(Combine)
export(Diff)
export(Expand)
export(GM2.lvm)
export(GM3.lvm)
export(Gamma.lvm)
export(Graph)
export(Grep)
export(IC)
export(IV)
export(Inverse)
export(Missing)
export(Model)
export(NA2x)
export(NR)
export(OR)
export(PD)
export(Print)
export(Range.lvm)
export(Ratio)
export(Sequence.lvm)
export(Specials)
export(Weights)
export(aalenExponential.lvm)
export(addhook)
export(addvar)
export(adjMat)
export(alpha_zmax)
export(ancestors)
export(as.sim)
export(backdoor)
export(baptize)
export(beta.lvm)
export(binary.lvm)
export(binomial.lvm)
export(binomial.rd)
export(binomial.rr)
export(blockdiag)
export(bootstrap)
export(cancel)
export(categorical)
export(children)
export(chisq.lvm)
export(click)
export(closed_testing)
export(colorbar)
export(colsel)
export(commutation)
export(compare)
export(complik)
export(confband)
export(confpred)
export(constant.lvm)
export(constrain)
export(constraints)
export(contr)
export(correlation)
export(covariance)
export(coxExponential.lvm)
export(coxGompertz.lvm)
export(coxWeibull.lvm)
export(csplit)
export(curly)
export(decomp.specials)
export(density.sim)
export(descendants)
export(describecoef)
export(devcoords)
export(diagtest)
export(distribution)
export(dmvn0)
export(dsep)
export(edgeList)
export(edgelabels)
export(endogenous)
export(equivalence)
export(estimate)
export(estimate.default)
export(eventTime)
export(exogenous)
export(expit)
export(finalize)
export(fixsome)
export(foldr)
export(forestplot)
export(fplot)
export(functional)
export(gaussian.lvm)
export(gaussian_logLik.lvm)
export(getSAS)
export(gethook)
export(getoutcome)
export(gkgamma)
export(gof)
export(graph2lvm)
export(heavytail)
export(id.lvm)
export(idplot)
export(igraph.lvm)
export(iid)
export(images)
export(index)
export(information)
export(intercept)
export(intervention)
export(kill)
export(ksmooth2)
export(latent)
export(lava)
export(lava.options)
export(loggamma.lvm)
export(logit)
export(logit.lvm)
export(lognormal.lvm)
export(lvm)
export(makemissing)
export(manifest)
export(mat.lvm)
export(measurement)
export(measurement.error)
export(mixture)
export(modelPar)
export(modelVar)
export(modelsearch)
export(moments)
export(multigroup)
export(multinomial)
export(multinomial.lvm)
export(mvn.lvm)
export(mvnmix)
export(na.pass0)
export(none.lvm)
export(nonlinear)
export(normal.lvm)
export(odds)
export(offdiag)
export(offdiags)
export(ones.lvm)
export(ordinal)
export(ordreg)
export(pairwise.diff)
export(parameter)
export(parents)
export(pareto.lvm)
export(parlabels)
export(parpos)
export(pars)
export(parsedesign)
export(partialcor)
export(path)
export(pcor)
export(pdfconvert)
export(plot.sim)
export(plotConf)
export(plot_region)
export(poisson.lvm)
export(predictlvm)
export(probit.lvm)
export(procformula)
export(randomslope)
export(regfix)
export(regression)
export(revdiag)
export(riskcomp)
export(rmvar)
export(rmvn0)
export(roots)
export(rot2D)
export(rot3D)
export(rotate2)
export(rsq)
export(scheffe)
export(score)
export(sim)
export(sinks)
export(spaghetti)
export(starter.multigroup)
export(startvalues)
export(startvalues0)
export(startvalues1)
export(startvalues2)
export(student.lvm)
export(summary.sim)
export(surface)
export(test_wald)
export(threshold.lvm)
export(tigol)
export(timedep)
export(toformula)
export(totaleffects)
export(tr)
export(trim)
export(twostage)
export(twostageCV)
export(uniform.lvm)
export(var_ic)
export(variance)
export(variances)
export(vars)
export(vec)
export(wait)
export(weibull.lvm)
export(wkm)
export(wrapvec)
export(x2NA)
export(zibreg)
importFrom(future.apply,future_lapply)
importFrom(future.apply,future_mapply)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,colors)
importFrom(grDevices,gray.colors)
importFrom(grDevices,heat.colors)
importFrom(grDevices,palette)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,topo.colors)
importFrom(grDevices,xy.coords)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,contour)
importFrom(graphics,contour.default)
importFrom(graphics,curve)
importFrom(graphics,identify)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,locator)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,rug)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(methods,as)
importFrom(methods,new)
importFrom(stats,AIC)
importFrom(stats,addmargins)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,confint.default)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,cov2cor)
importFrom(stats,density)
importFrom(stats,density.default)
importFrom(stats,deriv)
importFrom(stats,dnorm)
importFrom(stats,effects)
importFrom(stats,family)
importFrom(stats,fft)
importFrom(stats,formula)
importFrom(stats,get_all_vars)
importFrom(stats,glm)
importFrom(stats,influence)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.weights)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,nlminb)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,qchisq)
importFrom(stats,qf)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rchisq)
importFrom(stats,residuals)
importFrom(stats,rgamma)
importFrom(stats,rlnorm)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weights)
importFrom(survival,is.Surv)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,getFromNamespace)
importFrom(utils,getTxtProgressBar)
importFrom(utils,glob2rx)
importFrom(utils,head)
importFrom(utils,methods)
importFrom(utils,modifyList)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,stack)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
