LAVA
====

LAVA is a continuous integration system for deploying operating systems onto
physical and virtual hardware for running tests.

This repository contains the dispatcher and server source code.

Installation instructions
=========================

For latest information, see the lava-server documentation:
* [first installation](https://lava.readthedocs.io/en/latest/admin/basic-tutorials/instance/install.html)

Support
=======

Read the [contact](https://lava.readthedocs.io/en/latest/introduction/contact.html) page.

Reporting Bugs
==============

Bugs should first be discussed by subscribing to the
[lava-users](https://lists.lavasoftware.org/mailman3/lists/lava-users.lists.lavasoftware.org/)
mailing list.

Bug reports and new LAVA features are tracked using
[GitLab](https://gitlab.com/lava/lava/issues).
