Installation instructions
=======================

For latest information, see the lava-server documentation:
https://master.lavasoftware.org/static/docs/v2/first-installation.html
https://master.lavasoftware.org/static/docs/v2/installing_on_debian.html

Support
=======

https://master.lavasoftware.org/static/docs/v2/support.html

Reporting Bugs
==============

Bugs should first be discussed by subscribing to the
lava-users mailing list.
https://lists.linaro.org/mailman/listinfo/lava-users

Bug reports and new LAVA features are tracked using GitLab:
https://git.lavasoftware.org/lava/lava/issues
