LAVA
====

LAVA is an automated validation architecture primarily aimed at testing
deployments of systems based around the Linux kernel on ARM devices,
specifically ARMv7 and later.

This repository contains the dispatcher and server source code.

Installation instructions
=========================

For latest information, see the lava-server documentation:
* [first installation](https://docs.lavasoftware.org/lava/first-installation.html)
* [installing on Debian](https://docs.lavasoftware.org/lava/installing_on_debian.html)

Support
=======

Read the [support](https://docs.lavasoftware.org/lava/support.html) page.

Reporting Bugs
==============

Bugs should first be discussed by subscribing to the
[lava-users](https://lists.lavasoftware.org/mailman3/lists/lava-users.lists.lavasoftware.org/)
mailing list.

Bug reports and new LAVA features are tracked using
[GitLab](https://gitlab.com/lava/lava/issues).
