# First steps

To contribute to LAVA, start by reviewing the following guidelines:

- [Code of Conduct](./code-of-conduct.md)
- [Contribution guide](./methodology/contribute.md)
- [Contribution process](./tutorials/contributing.md)

Once you're familiar with the foundational basics, explore the development
guides to [setup](./development/setup.md) your environment,
[build](./development/build.md) and [test](./development/test.md) your changes.
