# Docker

## Docker deploy

Download the given docker image:

```yaml
actions:
- deploy:
    to: docker
    os: debian
    image: debian:sid
```

## Local images

The deploy action can use local docker images, just add `local: true`:

```yaml
actions:
- deploy:
    to: docker
    os: debian
    image:
      name: debian:sid
      local: true
```

## Docker login

To pull images from a password protected registry add a login section
with the username and password.

```yaml
actions:
- deploy:
    to: docker
    os: debian
    image:
      name: debian:sid
      login:
        user: foobar
        password: my_password
```

**Warning**: The pulled image will be cached on the local daemon and will
be available to other jobs running on same worker.
