---
hide:
  - navigation
---

# Welcome to LAVA

**LAVA** stands for **L**inaro **A**utomated **V**alidation **A**rchitecture.

LAVA is a *continuous integration* system for deploying operating systems onto
physical and virtual hardware for running tests.
Tests can be simple boot testing, bootloader testing and system level testing,
although extra hardware may be required for some system tests.
Results are tracked over time and data can be exported for further analysis.

## Documentation layout

The documentation is made of five sections:

* [Introduction](introduction/concepts.md)
* [User guide](user/first-steps.md)
* [Admin guide](admin/first-steps.md)
* [Developer guide](developer/first-steps.md)
* [Technical references](technical-references/architecture.md)

The guides are targeting specific audiences: Users, Admins or Developers.
Depending on how you expect to use LAVA, the **First steps** of each section
describe several useful starting points.

## Getting started

New to LAVA? Start with the
[Getting started](introduction/getting-started.md) guide.
