# First steps

These tutorials cover the essentials of managing your LAVA instance and lab.

| Tutorial | Description |
| -------- | ----------- |
| [Install](./basic-tutorials/instance/install.md) | Install a LAVA instance |
| [Configure](./basic-tutorials/instance/configure.md) | Configure a LAVA instance |
| [Upgrade](./basic-tutorials/instance/upgrade.md) | Upgrade a LAVA instance |
| [Backup](./basic-tutorials/instance/backup.md) | Back up a LAVA instance |
| [Permissions](./basic-tutorials/instance/permissions.md) | Manage user permissions |
| [Docker worker](./advanced-tutorials/docker-worker.md) | Install a LAVA Docker worker |
| [Device setup](basic-tutorials/device-setup/common.md) | Add devices to LAVA |

For the next steps, explore the advanced tutorials to learn more about LAVA
administration.
