# Mailing-lists

The best way to get in touch with the community and the LAVA developers is via
[mailing-lists](https://lists.lavasoftware.org).

## [lava-announce](mailto:lava-announce@lists.lavasoftware.org)

Subscribing to the **lava-announce** list is recommended for everyone using
LAVA, whether writing tests or viewing reports or administering a LAVA instance.

The release notes for each production release are sent to the **lava-announce**
mailing list.

Replies to this list are sent to the **lava-users** list.

## [lava-devel](mailto:lava-devel@lists.lavasoftware.org)

Subscribing to the **lava-devel** list is recommended for developers of LAVA.

**lava-devel** is aimed at supporting code contributors, device integration
engineers and instance admins who are working with the LAVA codebase.

Discussions about planning and new LAVA features also take place here.

## [lava-users](mailto:lava-users@lists.lavasoftware.org)

The **lava-users** mailing list concentrates on support for setting up and using
LAVA.

Subscribers include test writers, and individual admins. Users are encouraged to
contribute to answer queries from other users. Replies to the **lava-announce**
list are directed here.

## Discord

Our team is distributed globally, with members in Europe, North America, and
Asia. Join the LAVA Software Community Project on Discord for real-time
discussions and support.

Use this [invitation link](https://discord.com/invite/aUjpUFBvD2) to join our
server.
