# How to find issues

 When something is not working as expected, check the following areas:

* Job logs
* Job not terminating
* [Crashes in LAVA logs](../../technical-references/services/lava-worker.md)
* [State machine](../../technical-references/state-machine.md)
* Job queue not empty
* Performances
