# Health Checks

See [health checks](../../technical-references/configuration/health-check.md) for
adding device health checks.

[Notifications](../../technical-references/job-definition/notifications.md) should
be enabled to ensure administrators are alerted when health checks fail.
