# Interactive Tests

[Interactive](../../technical-references/job-definition/actions/test.md#interactive)
test action can be used for adding tests with a non-POSIX shell or just arbitrary
interactive applications. Examples are provided in the link.
