% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wait.R
\name{wait}
\alias{wait}
\alias{waitclick}
\title{Wait for user input (keyboard or mouse)}
\usage{
wait()
}
\description{
Wait for user input (keyboard or mouse)
}
\author{
Klaus K. Holst
}
\keyword{iplot}
