% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blockdiag.R
\name{blockdiag}
\alias{blockdiag}
\title{Combine matrices to block diagonal structure}
\usage{
blockdiag(x, ..., pad = 0)
}
\arguments{
\item{x}{matrix}

\item{\dots}{additional matrices}

\item{pad}{value outside block-diagonal}
}
\description{
Combine matrices to block diagonal structure
}
\examples{
A <- diag(3)+1
blockdiag(A,A,A,pad=NA)
}
\author{
Klaus K. Holst
}
