% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{bootstrap}
\alias{bootstrap}
\title{Generic bootstrap method}
\usage{
bootstrap(x, ...)
}
\arguments{
\item{x}{Model object}

\item{\dots}{Additional arguments}
}
\description{
Generic method for calculating bootstrap statistics
}
\seealso{
\code{bootstrap.lvm} \code{bootstrap.lvmfit}
}
\author{
Klaus K. Holst
}
