% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lava-package.R
\docType{package}
\name{lava-package}
\alias{lava-package}
\alias{lava}
\title{lava: Latent Variable Models}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

A general implementation of Structural Equation Models with latent variables (MLE, 2SLS, and composite likelihood estimators) with both continuous, censored, and ordinal outcomes (Holst and Budtz-Joergensen (2013) \doi{10.1007/s00180-012-0344-y}). Mixture latent variable models and non-linear latent variable models (Holst and Budtz-Joergensen (2020) \doi{10.1093/biostatistics/kxy082}). The package also provides methods for graph exploration (d-separation, back-door criterion), simulation of general non-linear latent variable models, and estimation of influence functions for a broad range of statistical models.

A general implementation of Structural Equation Models wth latent variables
(MLE, 2SLS, and composite likelihood estimators) with both continuous,
censored, and ordinal outcomes (Holst and Budtz-Joergensen (2013)
\url{doi:10.1007/s00180-012-0344-y}). Mixture latent variable models and
non-linear latent variable models (Holst and Budtz-Joergensen (2020)
\url{doi:10.1093/biostatistics/kxy082}). The package also provides methods for
graph exploration (d-separation, back-door criterion), simulation of general
non-linear latent variable models, and estimation of influence functions for
a broad range of statistical models.
}
\examples{
lava()
}
\seealso{
Useful links:
\itemize{
  \item \url{https://kkholst.github.io/lava/}
  \item \url{https://github.com/kkholst/lava}
  \item Report bugs at \url{https://github.com/kkholst/lava/issues}
}

}
\author{
\strong{Maintainer}: Klaus K. Holst \email{klaus@holst.it}

Authors:
\itemize{
  \item Klaus K. Holst \email{klaus@holst.it}
}

Other contributors:
\itemize{
  \item Benedikt Sommer [contributor]
  \item Brice Ozenne [contributor]
  \item Thomas Gerds [contributor]
}


Klaus K. Holst Maintainer: \href{mailto:klaus@holst.it}{klaus@holst.it}
}
\keyword{internal}
\keyword{package}
