% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lava-package.R
\docType{data}
\name{brisa}
\alias{brisa}
\title{Simulated data}
\format{
data.frame
}
\source{
Simulated
}
\description{
Simulated data
}
\keyword{datasets}
