% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lava-package.R
\docType{data}
\name{hubble2}
\alias{hubble2}
\title{Hubble data}
\format{
data.frame
}
\description{
Hubble data
}
\seealso{
hubble
}
\keyword{datasets}
