% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lava-package.R
\docType{data}
\name{nldata}
\alias{nldata}
\title{Example data (nonlinear model)}
\format{
data.frame
}
\source{
Simulated
}
\description{
Example data (nonlinear model)
}
\keyword{datasets}
