% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.default.R
\name{summary.sim}
\alias{summary.sim}
\title{Summary method for 'sim' objects}
\usage{
\method{summary}{sim}(
  object,
  estimate = NULL,
  se = NULL,
  confint,
  true = NULL,
  fun,
  names = NULL,
  unique.names = TRUE,
  minimal = FALSE,
  level = 0.95,
  quantiles = c(0, 0.025, 0.5, 0.975, 1),
  df = Inf,
  ...
)
}
\arguments{
\item{object}{sim object}

\item{estimate}{(optional) columns with estimates}

\item{se}{(optional) columns with standard error estimates}

\item{confint}{(optional) list of pairs of columns with confidence limits}

\item{true}{(optional) vector of true parameter values}

\item{fun}{(optional) summary function}

\item{names}{(optional) names of estimates}

\item{unique.names}{if TRUE, unique.names will be applied to column names}

\item{minimal}{if TRUE, minimal summary will be returned}

\item{level}{confidence level (0.95)}

\item{quantiles}{quantiles (0,0.025,0.5,0.975,1)}

\item{df}{degrees of freedom in t-distribution used for constructing CIs
(default Gaussian approximation)}

\item{...}{additional levels to lower-level functions}
}
\description{
Summary method for 'sim' objects
}
