% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lava-package.R
\docType{data}
\name{bmidata}
\alias{bmidata}
\title{Data}
\format{
data.frame
}
\description{
Description
}
\keyword{datasets}
