% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operators.R
\name{\%ni\%}
\alias{\%ni\%}
\alias{\%in.open\%}
\alias{\%in.closed\%}
\title{Matching operator (x not in y) oposed to the \code{\%in\%}-operator (x in y)}
\usage{
x \%ni\% y
}
\arguments{
\item{x}{vector}

\item{y}{vector of same type as \code{x}}
}
\value{
A logical vector.
}
\description{
Matching operator
}
\examples{

1:10 \%ni\% c(1,5,10)

}
\seealso{
\code{\link{match}}
}
\author{
Klaus K. Holst
}
\keyword{misc}
\keyword{utilities}
