% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operators.R
\name{\%++\%}
\alias{\%++\%}
\title{Concatenation operator}
\usage{
x \%++\% y
}
\arguments{
\item{x}{First object}

\item{y}{Second object of same class}
}
\description{
For matrices a block-diagonal matrix is created. For all other
data types he operator is a wrapper of \code{paste}.
}
\details{
Concatenation operator
}
\examples{
## Block diagonal
matrix(rnorm(25),5)\%++\%matrix(rnorm(25),5)
## String concatenation
"Hello "\%++\%" World"
## Function composition
f <- log \%++\% exp
f(2)
}
\seealso{
\code{blockdiag}, \code{\link{paste}}, \code{\link{cat}},
}
\author{
Klaus K. Holst
}
\keyword{misc}
\keyword{utilities}
