% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lava-package.R
\docType{data}
\name{semdata}
\alias{semdata}
\title{Example SEM data}
\format{
data.frame
}
\source{
Simulated
}
\description{
Simulated data
}
\keyword{datasets}
